package javax.swing.text.rtf;

import java.io.IOException;
import javax.swing.text.AttributeSet;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.StyleConstants;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Dictionary;

class RTFAttributes
{
    static RTFAttribute[] attributes;
    
    RTFAttributes() {
    }
    
    static Dictionary<String, RTFAttribute> attributesByKeyword() {
        Hashtable hashtable = new Hashtable(RTFAttributes.attributes.length);
        for (RTFAttribute rtfAttribute : RTFAttributes.attributes) {
            hashtable.put(rtfAttribute.rtfName(), rtfAttribute);
        }
        return hashtable;
    }
    
    static {
        Vector vector = new Vector();
        int n = 0;
        int n2 = 1;
        int n3 = 3;
        int n4 = 4;
        Boolean value = Boolean.valueOf(true);
        Boolean value2 = Boolean.valueOf(false);
        vector.addElement(new RTFAttributes.BooleanAttribute(n, StyleConstants.Italic, "i"));
        vector.addElement(new RTFAttributes.BooleanAttribute(n, StyleConstants.Bold, "b"));
        vector.addElement(new RTFAttributes.BooleanAttribute(n, StyleConstants.Underline, "ul"));
        vector.addElement(RTFAttributes.NumericAttribute.NewTwips(n2, StyleConstants.LeftIndent, "li", 0.0f, 0));
        vector.addElement(RTFAttributes.NumericAttribute.NewTwips(n2, StyleConstants.RightIndent, "ri", 0.0f, 0));
        vector.addElement(RTFAttributes.NumericAttribute.NewTwips(n2, StyleConstants.FirstLineIndent, "fi", 0.0f, 0));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n2, StyleConstants.Alignment, "ql", 0));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n2, StyleConstants.Alignment, "qr", 2));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n2, StyleConstants.Alignment, "qc", 1));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n2, StyleConstants.Alignment, "qj", 3));
        vector.addElement(RTFAttributes.NumericAttribute.NewTwips(n2, StyleConstants.SpaceAbove, "sa", 0));
        vector.addElement(RTFAttributes.NumericAttribute.NewTwips(n2, StyleConstants.SpaceBelow, "sb", 0));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n4, "tab_alignment", "tqr", 1));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n4, "tab_alignment", "tqc", 2));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n4, "tab_alignment", "tqdec", 4));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n4, "tab_leader", "tldot", 1));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n4, "tab_leader", "tlhyph", 2));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n4, "tab_leader", "tlul", 3));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n4, "tab_leader", "tlth", 4));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n4, "tab_leader", "tleq", 5));
        vector.addElement(new RTFAttributes.BooleanAttribute(n, "caps", "caps"));
        vector.addElement(new RTFAttributes.BooleanAttribute(n, "outl", "outl"));
        vector.addElement(new RTFAttributes.BooleanAttribute(n, "scaps", "scaps"));
        vector.addElement(new RTFAttributes.BooleanAttribute(n, "shad", "shad"));
        vector.addElement(new RTFAttributes.BooleanAttribute(n, "v", "v"));
        vector.addElement(new RTFAttributes.BooleanAttribute(n, "strike", "strike"));
        vector.addElement(new RTFAttributes.BooleanAttribute(n, "deleted", "deleted"));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n3, "saveformat", "defformat", "RTF"));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n3, "landscape", "landscape"));
        vector.addElement(RTFAttributes.NumericAttribute.NewTwips(n3, "paperw", "paperw", 12240));
        vector.addElement(RTFAttributes.NumericAttribute.NewTwips(n3, "paperh", "paperh", 15840));
        vector.addElement(RTFAttributes.NumericAttribute.NewTwips(n3, "margl", "margl", 1800));
        vector.addElement(RTFAttributes.NumericAttribute.NewTwips(n3, "margr", "margr", 1800));
        vector.addElement(RTFAttributes.NumericAttribute.NewTwips(n3, "margt", "margt", 1440));
        vector.addElement(RTFAttributes.NumericAttribute.NewTwips(n3, "margb", "margb", 1440));
        vector.addElement(RTFAttributes.NumericAttribute.NewTwips(n3, "gutter", "gutter", 0));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n2, "widowctrl", "nowidctlpar", value2));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n2, "widowctrl", "widctlpar", value));
        vector.addElement(new RTFAttributes.AssertiveAttribute(n3, "widowctrl", "widowctrl", value));
        RTFAttribute[] array = new RTFAttribute[vector.size()];
        vector.copyInto(array);
        RTFAttributes.attributes = array;
    }
    
    abstract static class GenericAttribute
    {
        int domain;
        Object swingName;
        String rtfName;
        
        protected GenericAttribute(int domain, Object swingName, String rtfName) {
            this.domain = domain;
            this.swingName = swingName;
            this.rtfName = rtfName;
        }
        
        public int domain() {
            return this.domain;
        }
        
        public Object swingName() {
            return this.swingName;
        }
        
        public String rtfName() {
            return this.rtfName;
        }
        
        abstract boolean set(MutableAttributeSet p0);
        
        abstract boolean set(MutableAttributeSet p0, int p1);
        
        abstract boolean setDefault(MutableAttributeSet p0);
        
        public boolean write(AttributeSet set, RTFGenerator rtfGenerator, boolean b) throws IOException {
            return this.writeValue(set.getAttribute(this.swingName), rtfGenerator, b);
        }
        
        public boolean writeValue(Object o, RTFGenerator rtfGenerator, boolean b) throws IOException {
            return false;
        }
    }
    
    static class BooleanAttribute extends RTFAttributes.GenericAttribute implements RTFAttribute
    {
        boolean rtfDefault;
        boolean swingDefault;
        protected static final Boolean True;
        protected static final Boolean False;
        
        public BooleanAttribute(int n, Object o, String s, boolean swingDefault, boolean rtfDefault) {
            super(n, o, s);
            this.swingDefault = swingDefault;
            this.rtfDefault = rtfDefault;
        }
        
        public BooleanAttribute(int n, Object o, String s) {
            super(n, o, s);
            this.swingDefault = false;
            this.rtfDefault = false;
        }
        
        @Override
        public boolean set(MutableAttributeSet set) {
            set.addAttribute(this.swingName, RTFAttributes.BooleanAttribute.True);
            return true;
        }
        
        @Override
        public boolean set(MutableAttributeSet set, int n) {
            set.addAttribute(this.swingName, (n != 0) ? RTFAttributes.BooleanAttribute.True : RTFAttributes.BooleanAttribute.False);
            return true;
        }
        
        @Override
        public boolean setDefault(MutableAttributeSet set) {
            if (this.swingDefault != this.rtfDefault || set.getAttribute(this.swingName) != null) {
                set.addAttribute(this.swingName, Boolean.valueOf(this.rtfDefault));
            }
            return true;
        }
        
        @Override
        public boolean writeValue(Object o, RTFGenerator rtfGenerator, boolean b) throws IOException {
            Boolean value;
            if (o == null) {
                value = Boolean.valueOf(this.swingDefault);
            }
            else {
                value = (Boolean)o;
            }
            if (b || (boolean)value != this.rtfDefault) {
                if ((boolean)value) {
                    rtfGenerator.writeControlWord(this.rtfName);
                }
                else {
                    rtfGenerator.writeControlWord(this.rtfName, 0);
                }
            }
            return true;
        }
        
        static {
            True = Boolean.valueOf(true);
            False = Boolean.valueOf(false);
        }
    }
    
    static class AssertiveAttribute extends RTFAttributes.GenericAttribute implements RTFAttribute
    {
        Object swingValue;
        
        public AssertiveAttribute(int n, Object o, String s) {
            super(n, o, s);
            this.swingValue = Boolean.valueOf(true);
        }
        
        public AssertiveAttribute(int n, Object o, String s, Object swingValue) {
            super(n, o, s);
            this.swingValue = swingValue;
        }
        
        public AssertiveAttribute(int n, Object o, String s, int i) {
            super(n, o, s);
            this.swingValue = Integer.valueOf(i);
        }
        
        @Override
        public boolean set(MutableAttributeSet set) {
            if (this.swingValue == null) {
                set.removeAttribute(this.swingName);
            }
            else {
                set.addAttribute(this.swingName, this.swingValue);
            }
            return true;
        }
        
        @Override
        public boolean set(MutableAttributeSet set, int n) {
            return false;
        }
        
        @Override
        public boolean setDefault(MutableAttributeSet set) {
            set.removeAttribute(this.swingName);
            return true;
        }
        
        @Override
        public boolean writeValue(Object o, RTFGenerator rtfGenerator, boolean b) throws IOException {
            if (o == null) {
                return !b;
            }
            if (o.equals(this.swingValue)) {
                rtfGenerator.writeControlWord(this.rtfName);
                return true;
            }
            return !b;
        }
    }
    
    static class NumericAttribute extends RTFAttributes.GenericAttribute implements RTFAttribute
    {
        int rtfDefault;
        Number swingDefault;
        float scale;
        
        protected NumericAttribute(int n, Object o, String s) {
            super(n, o, s);
            this.rtfDefault = 0;
            this.swingDefault = null;
            this.scale = 1.0f;
        }
        
        public NumericAttribute(int n, Object o, String s, int i, int n2) {
            this(n, o, s, Integer.valueOf(i), n2, 1.0f);
        }
        
        public NumericAttribute(int n, Object o, String s, Number swingDefault, int rtfDefault, float scale) {
            super(n, o, s);
            this.swingDefault = swingDefault;
            this.rtfDefault = rtfDefault;
            this.scale = scale;
        }
        
        public static RTFAttributes.NumericAttribute NewTwips(int n, Object o, String s, float value, int n2) {
            return new RTFAttributes.NumericAttribute(n, o, s, new Float(value), n2, 20.0f);
        }
        
        public static RTFAttributes.NumericAttribute NewTwips(int n, Object o, String s, int n2) {
            return new RTFAttributes.NumericAttribute(n, o, s, null, n2, 20.0f);
        }
        
        @Override
        public boolean set(MutableAttributeSet set) {
            return false;
        }
        
        @Override
        public boolean set(MutableAttributeSet set, int i) {
            Integer value;
            if (this.scale == 1.0f) {
                value = Integer.valueOf(i);
            }
            else {
                value = new Float((float)i / this.scale);
            }
            set.addAttribute(this.swingName, value);
            return true;
        }
        
        @Override
        public boolean setDefault(MutableAttributeSet set) {
            Number swingDefault = (Number)set.getAttribute(this.swingName);
            if (swingDefault == null) {
                swingDefault = this.swingDefault;
            }
            if (swingDefault != null && ((this.scale == 1.0f && swingDefault.intValue() == this.rtfDefault) || Math.round(swingDefault.floatValue() * this.scale) == this.rtfDefault)) {
                return true;
            }
            this.set(set, this.rtfDefault);
            return true;
        }
        
        @Override
        public boolean writeValue(Object o, RTFGenerator rtfGenerator, boolean b) throws IOException {
            Number swingDefault = (Number)o;
            if (swingDefault == null) {
                swingDefault = this.swingDefault;
            }
            if (swingDefault == null) {
                return true;
            }
            int round = Math.round(swingDefault.floatValue() * this.scale);
            if (b || round != this.rtfDefault) {
                rtfGenerator.writeControlWord(this.rtfName, round);
            }
            return true;
        }
    }
}
